<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00049000AgrupamentoContratosFaturamentoFactory;
use Teknisa\SM\Service\Sm00004000ContratoService;
use Teknisa\Libs\Util\NewCode;

class Sm00049000AgrupamentoContratosFaturamentoService {
    protected $agrupamentoFactory;
    protected $contratoService;
    protected $newCode;
    
    public function __construct(Sm00049000AgrupamentoContratosFaturamentoFactory $agrupamentoFactory, Sm00004000ContratoService $contratoService, NewCode $newCode){
        $this->agrupamentoFactory = $agrupamentoFactory;
        $this->contratoService  = $contratoService;
        $this->newCode          = $newCode;
    }
    
    /**
     * Faz a validação dos contratos que dejam ser agrupados
    **/
    public function validarContratosAgrupamento($contratos, $nrorg, $nragrupamento){
        $msgError       = null;
        $nrseqcontratos = array();
        foreach($contratos as $contrato)
            array_push($nrseqcontratos, $contrato["NRSEQCONTRATO"]);
        
        $nrseqcontratos = implode(",", $nrseqcontratos);
        $params         = array(
            "NRAGRUPAMENTO" => $nragrupamento,
            "NRORG"         => $nrorg
        );
        
        $paramContratos = $this->agrupamentoFactory->getParametrosFatContratos($nrseqcontratos, $params);
        if(count($paramContratos) > 1)
            throw new \Exception("PARC_DIFERENTES");
            
        return null;
    }  
    
    public function salvaContratosAgrupamento($nrseqcontratos, $nragrupamento, $nrorg, $operador){
        foreach($nrseqcontratos as $nrseqcontrato) {
            $situacaoContr = $nrseqcontrato["IDSITUACAOCONTRATO"];
            $contrato      = $nrseqcontrato["NRSEQCONTRATO"];
                
            /* Insere o Contrato no Agrupamento */
            $nragrupamentocont = $this->newCode->getCodeByTableName("FCT_RELACAGRUPAMENTOCONT", $nrorg, 1, 10, true);
            $params = array(
                "NRAGRUPAMENTOCONT" => $nragrupamentocont,
                "NRAGRUPAMENTO"     => $nragrupamento,
                "NRSEQCONTRATO"     => $contrato,
                "NRORG"             => $nrorg,
                "OPERADOR"          => $operador
            );
            
            $this->agrupamentoFactory->insereContrAgrup($params);

            /* Se o contrato estiver aprovado, regera as parcelas */
            if($situacaoContr == 'A')
                $this->contratoService->executarProcedureFctGerarParcelasFatura($contrato, $nrorg, $operador);
        }
    }
    
    /**
     * Valida se os escopos possuem informações similares
    **/
    public function validarEscoposAgrupamento($escopos, $nrorg) {
        $strEscopos         = implode(",", $escopos);
        $paramsFaturaEscopo = $this->agrupamentoFactory->getParametrosFaturamentoEscopo($strEscopos, $nrorg);
        
        if(count($paramsFaturaEscopo) > 1) {
            throw new \Exception("PARAMS_DIFERENTE");
        } else {
            $competencias = $this->agrupamentoFactory->getCompetenciasFaturamentoEscopo($strEscopos, $nrorg);
            foreach($competencias as $competencia) {
                foreach($escopos as $escopo) {
                    if(strstr($competencia["NRTIPOORCAMENTO"], '_' . $escopo . '_') == false)
                        throw new \Exception("COMPET_DIFERENTE");
                }
            }
            
            $paramsDoServico = $this->agrupamentoFactory->getParametrosFaturamentoServicoDoEscopo($strEscopos, $nrorg);
            if (count($paramsDoServico) > 1) {
                return false;
            } else {
                $competenciasServico = $this->agrupamentoFactory->getCompetenciasFaturamentoServicoDoEscopo($strEscopos, $nrorg);
                foreach($competenciasServico as $competencia) {
                    foreach($escopos as $escopo) {
                        if(strstr($competencia["NRTIPOORCAMENTO"], '_' . $escopo . '_') == false)
                            return false;
                    }
                }
                
                return true;
            }
        }
    }
    
    /**
     * retorna as informações de faturamento do escopo
    **/
    public function getInformacoesEscopos($escopos, $nrorg) {
        $escopos            = implode(",", $escopos);
        $paramsFaturaEscopo = $this->agrupamentoFactory->getParametrosFaturamentoEscopo($escopos, $nrorg);
        
        return $paramsFaturaEscopo[0];
    }
    
    /**
     * Salva os escopos ligados ao agrupamentos
    **/
    public function salvarEscoposAgrupamento($escopos, $nragrupamento, $cdoperador, $nrorg) {
        foreach($escopos as $escopo) {
            $nrescopoagrup = $this->newCode->getCodeByTableName("FCT_RELACESCOPOAGRUPAMENTO", $nrorg, 1, 10, true);
            $params = array(
                "NRRELACESCAGRUP" => $nrescopoagrup,
                "NRTIPOORCAMENTO" => $escopo,
                "NRAGRUPAMENTO"   => $nragrupamento,
                "NRORG"           => $nrorg,
                "NRORGINCLUSAO"   => $nrorg,
                "CDOPERINCLUSAO"  => $cdoperador,
                "IDATIVO"         => "S"
            );
            
            $this->agrupamentoFactory->salvarEscoposAgrupamento($params);
        }
    }
}